"""
Get song info.
"""
import shutil
import select
from pkg_resources import iter_entry_points

import mpd

from . import brainz
from . import util


def get_client_types():
    """Enumerate the pidi.plugin.client entry point and return installed client types."""
    client_types = {
        'mpd': ClientMPD
    }

    for entry_point in iter_entry_points("pidi.plugin.client"):
        try:
            plugin = entry_point.load()
            client_types[plugin.option_name] = plugin
        except (ModuleNotFoundError, ImportError) as err:
            print("Error loading client plugin {entry_point}: {err}".format(
                entry_point=entry_point,
                err=err
            ))

    return client_types


class ClientMPD():
    """Client for MPD and MPD-like (such as Mopidy) music back-ends."""
    def __init__(self, port=6600, server="localhost"):
        """Initialize mpd."""
        self._client = mpd.MPDClient()

        try:
            self._client.connect(server, port)

        except ConnectionRefusedError:
            raise RuntimeError("error: Connection refused to mpd/mopidy.")

        self._client.send_idle('player')

    def add_args(argparse):  # pylint: disable=no-self-argument
        """Expand argparse instance with client-specific args."""

    def currentsong(self):
        """Return current song details."""
        self._client.noidle()
        result = self._client.currentsong()  # pylint: disable=no-member
        self._client.send_idle('player')
        return result

    def status(self):
        """Return current status details."""
        self._client.noidle()
        result = self._client.status()  # pylint: disable=no-member
        self._client.send_idle('player')
        return result

    def update_pending(self, timeout=0.1):
        """Determine if anything has changed on the server."""
        result = select.select([self._client], [], [], timeout)[0]
        return self._client in result

    def get_art(self, cache_dir, size):
        """Get the album art."""
        song = self.currentsong()
        if len(song) < 2:
            print("album: Nothing currently playing.")
            util.bytes_to_file(util.default_album_art(), cache_dir / "current.jpg")
            return

        artist = song.get('artist')
        title = song.get('title')
        album = song.get('album', title)
        file_name = "{artist}_{album}_{size}.jpg".format(
            artist=artist,
            album=album,
            size=size
        ).replace("/", "")
        file_name = cache_dir / file_name

        if file_name.is_file():
            shutil.copy(file_name, cache_dir / "current.jpg")
            print("album: Found cached art.")

        else:
            print("album: Downloading album art...")

            brainz.init()
            album_art = brainz.get_cover(song, size)

            if not album_art:
                album_art = util.default_album_art()

            util.bytes_to_file(album_art, cache_dir / file_name)
            util.bytes_to_file(album_art, cache_dir / "current.jpg")

            print("album: Swapped art to {artist}, {title}.".format(
                artist=artist,
                title=title
            ))



"""pidi - setup.py"""
import sys
import setuptools

try:
    import pidi
except (ImportError, SyntaxError):
    print("error: pidi requires Python 3.6 or greater.")
    sys.exit(1)


try:
    import pypandoc
    LONG_DESC = pypandoc.convert("README.md", "rst")
except(IOError, ImportError, RuntimeError):
    LONG_DESC = open('README.md').read()


VERSION = pidi.__version__
DOWNLOAD = "https://github.com/pimoroni/pidi/archive/%s.tar.gz" % VERSION


setuptools.setup(
    name="pidi",
    version=VERSION,
    author="Phil Howard",
    author_email="phil@pimoroni.com",
    description="Download and display album art for mpd tracks.",
    long_description=LONG_DESC,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/pimoroni/pidi",
    download_url=DOWNLOAD,
    classifiers=[
        "Environment :: X11 Applications",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.6",
    ],
    packages=["pidi"],
    entry_points={
        "console_scripts": ["pidi=pidi.__main__:main"]
    },
    install_requires=[
        "musicbrainzngs",
        "python-mpd2",
    ],
    extras_require={
        'mpv': ["python-mpv"],
        'tk': ["tk", "pil"],
    },
    python_requires=">=3.6",
    test_suite="tests",
    include_package_data=True
)



"""
Display Output Classes.
"""
from pkg_resources import iter_entry_points


def get_display_types():
    """Enumerate the pidi.plugin.display entry point and return installed display types."""
    display_types = {
        'dummy': DisplayDummy,
        'mpv': DisplayMPV,
    }

    for entry_point in iter_entry_points("pidi.plugin.display"):
        try:
            plugin = entry_point.load()
            display_types[plugin.option_name] = plugin
        except (ModuleNotFoundError, ImportError) as err:
            print("Error loading display plugin {entry_point}: {err}".format(
                entry_point=entry_point,
                err=err
            ))

    return display_types


class Display():
    """Base class to represent a pidi display output."""
    # pylint: disable=too-many-instance-attributes
    def __init__(self, args=None):
        """Initialise a new display."""
        self._size = args.size
        self._title = ''
        self._shuffle = False
        self._repeat = False
        self._state = ''
        self._volume = 0
        self._progress = 0
        self._elapsed = 0

        self._title = ''
        self._album = ''
        self._artist = ''

    def update_album_art(self, input_file):
        """Update the display album art."""
        raise NotImplementedError

    def update_overlay(self, shuffle, repeat, state, volume,
                       progress, elapsed, title, album, artist):
        """Update the display transport information."""
        # pylint: disable=too-many-arguments
        self._shuffle = shuffle
        self._repeat = repeat
        self._state = state
        self._volume = volume
        self._progress = progress
        self._elapsed = elapsed
        self._title = title
        self._album = album
        self._artist = artist

    def redraw(self):
        """Redraw the display."""
        raise NotImplementedError

    def add_args(argparse):  # pylint: disable=no-self-argument
        """Expand argparse instance with display-specific args."""


class DisplayDummy(Display):
    """Dummy display output. Does nothing."""
    def update_album_art(self, input_file):
        """Ignore display album art. This is a dummy."""

    def redraw(self):
        """Ignore redraw calls. This is a dummy."""


class DisplayMPV(Display):
    """Display using MPV, playing album art as if it were media."""
    def __init__(self, args):
        """Initialise an MPV display."""
        import mpv  # pylint: disable=import-outside-toplevel
        Display.__init__(self, args)
        self._player = mpv.MPV(start_event_thread=False)
        self._player["force-window"] = "immediate"
        self._player["keep-open"] = "yes"
        self._player["geometry"] = "{size}x{size}".format(size=self._size)
        self._player["autofit"] = "{size}x{size}".format(size=self._size)
        self._player["title"] = "pidi"
        self._art = None

    def update_album_art(self, input_file):
        """Update album art."""
        self._art = str(input_file)

    def redraw(self):
        """Display album art using MPV"""
        if self._art is not None:
            self._player.player(self._art)
